
function driver_func_kde_naps_nmem(distribution_vector,names,trials, min_pow,max_pow,cpu_n,cpp_code)

% include function files
addpath("functions/")

% JOB ARRAY VARIABLES -----------------------------------------------------

% cast characters to strings or to ints for relevant variables
distribution_vector = string(distribution_vector)
names = string(names)
trials = str2num(trials)
min_pow = str2num(min_pow)
max_pow = str2num(max_pow)
cpu_n = str2num(cpu_n)
cpp_code = string(cpp_code)

% %%%%%%%%%%%%%%%%% COMMENT OUT IF RUNNING LOCALLY %%%%%%%%%%%%%%%%%%%%%%%%%%

% % create parallel process on cluster
% 
% % create a local cluster object
% pc = parcluster('local')
%  
% % explicitly set the JobStorageLocation to the temp directory that was
% % created in your sbatch script
% pc.JobStorageLocation = strcat('/home/zmerino/.matlab/temp_cluster_jobs/', getenv('SLURM_JOB_ID'))
% 
% % start the matlabpool with maximum available workers
% % control how many workers by setting ntasks in your sbatch script
% poolobj = gcp('nocreate'); % get the pool object, and do it avoiding creating a new one.
% if isempty(poolobj) % check if there is not a pool.
%     % create parallel process with given parameters
%     parpool(pc, str2num(getenv('SLURM_CPUS_ON_NODE')), 'IdleTimeout', Inf)
% else
%     delete( gcp('nocreate')); % delete the current pool object.
%     % create parallel process with given parameters
%     parpool(pc, str2num(getenv('SLURM_CPUS_ON_NODE')), 'IdleTimeout', Inf)
% end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% add C++ code mex file to workspace
addpath(cpp_code)
parent_dir = 'data_kde'
% location to save data
write_dir = fullfile(parent_dir,sprintf('%s_cpu_%d_t_%d', distribution_vector, cpu_n, trials))
% name of data table to save
table_name = sprintf('%s_cpu_%d_t_%d.dat', distribution_vector, cpu_n, trials)

% make directory if doesn't exist
status = mkdir(write_dir);

% class assignment
actual = distributions;
actual.generate_data = false;

% User Options ============================================================
% script switching board
data_type_flag =            true;   %<- true/false integer powers of 2/real powers of 2
% rndom data generation parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
step =                      1;  %<---- control synthetic rndom samples to skip being created
temp_min_limit =            0; %<---- set upper limit for both
actual.min_limit =          temp_min_limit;  %<--- lower limit to plot
temp_max_limit =            1000; %<---- set upper limit for both
actual.max_limit =          temp_max_limit; %<--- upper limit to plot
x_resolution =              1000;
% changes how data files are accessed when using data generated from the
% Univariant Random Sample Generator available on zmerino's github
cpu_type =                   '\';%<--- '\' or '/' for windows or linux

% distribution_vector = ["Trimodal-Normal","Uniform","Normal","Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5","Generalized-Pareto","Stable","Stable2","Stable3"];
distribution = distribution_vector';
% names = ["Tri-Modal-Normal","Uniform", "Normal", "Beta(0.5,1.5)", "Beta(2,0.5)", "Beta(0.5,0.5)", "Generalized-Pareto","Stable","Stable2","Stable3"];

% distribution_vector = ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5","Generalized-Pareto"];
% distribution = distribution_vector';
% names = [ "Beta(0.5,1.5)", "Beta(2,0.5)", "Beta(0.5,0.5)", "Generalized-Pareto"];


% find amy of the strings in "str" inside of "distribtuionVector"
str = ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5"];
flag = zeros(1,length(distribution_vector));
for a = 1:size(distribution_vector,2)
    for b = 1:size(str,2)
        if strcmp(distribution_vector(a),str(b))
            flag(a) = 1;
            break
        else
            flag(a) = 0;
        end
    end
end

% only works for division with no remainders
fail_nmem = zeros((max_pow-min_pow+1)/step, trials,length(distribution_vector));
fail_naps_parallel = zeros((max_pow-min_pow+1)/step, trials,length(distribution_vector));
fail_naps_serial = zeros((max_pow-min_pow+1)/step, trials,length(distribution_vector));
lagrange_naps_parallel = zeros((max_pow-min_pow+1)/step, trials,length(distribution_vector));
avg_lagrange_naps_parallel = zeros((max_pow-min_pow+1)/step, trials,length(distribution_vector));
lagrange_naps_serial = zeros((max_pow-min_pow+1)/step, trials,length(distribution_vector));
lagrange_nmem = zeros((max_pow-min_pow+1)/step, trials,length(distribution_vector));

% table to store meta data
global_table = table();
mse_table = table();

% change code to multithreaded version
for j = 1:length(distribution_vector)
    % Define plot vector for dist_list from 0-1
    if flag(j)
        actual.min_limit = 0;
        actual.max_limit = 1;
        actual.x = linspace(actual.min_limit,actual.max_limit,x_resolution); %<----******** need to update code
    else
        % Define plot vector for distribution from actual.min_limit-actual.max_limit
        actual.min_limit = 0;
        actual.max_limit = 10;
        actual.x = linspace(actual.min_limit,actual.max_limit,x_resolution);
    end
    % Current distribution name
    actual.dist_name = distribution_vector(j);
    % file name for actual distribution. "A_" puts at the top of the folder.
    actual.filename = sprintf(['A_', char(actual.dist_name),'_Act']);

    % creat rndom object
    rndom = actual;

    T_track = zeros(max_pow-min_pow,1);
    BR_track = zeros(max_pow-min_pow,trials);
    BR0_track = zeros(max_pow-min_pow,trials);
    sample_data = zeros(max_pow-min_pow,1);
    cpu_time_se = [];
    cpu_time_nmem = [];
    sample_track = [];

    % Create vector of  samples
    sample_vec = utils.sample_pow(min_pow,max_pow,data_type_flag,step);

    cpu_vec_se_parallel = zeros(length(sample_vec),trials);
    cpu_vec_se_serial = zeros(length(sample_vec),trials);
    cpu_vec_nmem = zeros(length(sample_vec),trials);
    cpu_se_kde = zeros(length(sample_vec),trials);
    cpu_kde = zeros(length(sample_vec),trials);

    wall_vec_se_parallel = zeros(length(sample_vec),trials);
    wall_vec_se_serial = zeros(length(sample_vec),trials);
    wall_vec_nmem = zeros(length(sample_vec),trials);
    wall_se_kde = zeros(length(sample_vec),trials);
    wall_kde = zeros(length(sample_vec),trials);

    kl_vec_se = zeros(length(sample_vec),trials);
    kl_vec_nmem = zeros(length(sample_vec),trials);
    mse_vec_se = zeros(length(sample_vec),trials);
    mse_vec_nmem = zeros(length(sample_vec),trials);
    block_scale = zeros(4,length(sample_vec),trials);
    block_size = zeros(4,length(sample_vec),trials);

    % store analysis.mse per block for all trials per distribution and sample size
    mse_dists_naps = cell(length(sample_vec),trials);
    mse_dists_nmem = cell(length(sample_vec),trials);

    % store block size and scale per distribution and sample size
    block_scale_all = cell(length(sample_vec),trials);
    block_size_all = cell(length(sample_vec),trials);

    actual = actual.dist_list();

    for k = 1:length(sample_vec)

        % generate empty cell arrays to save pdf estiamtes and samples

        naps_pdf_data = {};
        nmem_pdf_data = {};
        kde_pdf_data = {};
        se_kde_pdf_data = {};

        for i = 1:trials

            % initialize to 0. used to count number of failed NMEM pdfe
            fail_flag = 0;

            rndom.Ns = sample_vec(k);
            realx = linspace(actual.min_limit,actual.max_limit,rndom.Ns);
            % p-vector definition for Rtree
            p = [1,0.55,1,0.33,2,ceil(0.0625*rndom.Ns^0.5),40];

            % Create rndom.filename for each distribtuion
            rndom.filename = sprintf(['D_', char(actual.dist_name),'_T_','%d', '_S_','%d'],i, rndom.Ns);

            %==========================================================
            % % % % % % % start of estimates % % % % % % % % %
            %==========================================================
            tic

            %-- NAP start
            rndom.randomVSactual = "random";
            rndom = dist_list(rndom);
            sample = rndom.rndData;

            % cpu time
            tintialSE = cputime;
            % wall time
            tic
            % NAP PARALLEL ------------------------------------------------
            % naps object instantiation
            naps = NAPS;
            naps.max_bs = 1e3;
            serial = false;
            naps = naps.stitch(sample, serial);

            % extract relevant parameters from object after stich() method
            fail_code = naps.failed;
            SE_x = naps.sx;
            SE_pdf = naps.sPDF;
            SE_cdf = naps.sCDF;
            SE_u = naps.u;
            SE_SQR = naps.sqr;
            nBlocks = naps.nBlocks;
            rndom.Ns = naps.N;
            binrndom.Ns =  naps.binN;
            LG = naps.LG;
            max_LG = naps.LG_max;
            sum_LG = naps.LG_sum;
            T = naps.T;
            BRlevel = naps.BRlevel;
            BR0 = naps.BR0;

            tcpuSE_parallel = cputime-tintialSE;
            twallSE_parallel = toc;
            fail_naps_parallel(k,i,j) = fail_code;

            % cpu time
            tintialSE = cputime;
            % wall time
            tic
            % NAP SERIAL --------------------------------------------------
            % naps object instantiation
            naps_serial = NAPS;
            naps_serial.max_bs = 1e3;
            serial = true;
            naps_serial = naps.stitch(sample, serial);

            % extract relevant parameters from object after stich() method
            fail_code = naps_serial.failed;
            %             SE_x = naps.sx;
            %             SE_pdf = naps.sPDF;
            %             SE_cdf = naps.sCDF;
            %             SE_u = naps.u;
            %             SE_SQR = naps.sqr;
            %             nBlocks = naps.nBlocks;
            %             rndom.Ns = naps.N;
            %             binrndom.Ns =  naps.binN;
            %             LG = naps.LG;
            %             max_LG = naps.LG_max;
            %             sum_LG = naps.LG_sum;
            %             T = naps.T;
            %             BRlevel = naps.BRlevel;
            %             BR0 = naps.BR0;

            tcpuSE_serial = cputime-tintialSE;
            twallSE_serial = toc;

            fail_naps_serial(k,i,j) = fail_code;

            % store information about block
            block_size(1,k,i) = max(naps.block_size);
            block_size(2,k,i) = min(naps.block_size);
            block_size(3,k,i) = mean(naps.block_size);
            block_size(4,k,i) = median(naps.block_size);

            block_scale(1,k,i) = max(naps.block_scale);
            block_scale(2,k,i) = min(naps.block_scale);
            block_scale(3,k,i) = mean(naps.block_scale);
            block_scale(4,k,i) = median(naps.block_scale);

            % NMEM --------------------------------------------------------
            try
                % cpu time
                tintialNMEM = cputime;
                % wall time
                tic
                % [FAILED, XI, F, CDF, SQR, LAGRANGE, SCORE, CONFIDENCE, SURD] = EstimatePDF(X)
                [failed, x_NMEM, pdf_NMEM, cdf_NMEM,sqr_NMEM, lagrange_multipler,score, confidence, surd] = EstimatePDF(sample);
                fail_nmem(k,i,j) = 0;
                tcpuNMEM = cputime-tintialNMEM;
                twallNMEM = toc;

                n = length(sqr_NMEM);
                dx = 1 / (n + 1);
                u_NMEM = dx:dx:(n * dx);
            catch
                warning('Problem using function.  Assigning a value of 0.');
                lagrange_multipler = 0;
                x_NMEM = linspace(min(sample),max(sample),length(sample))';
                pdf_NMEM = 0*ones(length(sample),1);
                cdf_NMEM = 0*ones(length(sample),1);
                fail_nmem(k,i,j) = 1;
                % don't record cpu time if estimator failed
                tcpuNMEM = NaN;
                twallNMEM = NaN;
            end


            % KDE ---------------------------------------------------------
            tintialKDE = cputime;
            tic
            [pdf_KDE,x_KDE] = ksdensity(sample);
            twallKDE = toc;
            tcpuKDE = cputime-tintialKDE;

            % NAP w/ KDE --------------------------------------------------

            rndom.randomVSactual = "random";
            rndom = dist_list(rndom);
            sample = rndom.rndData;

            tintialSEKDE = cputime;
            tic

            % naps object instantiation
            naps_kde = NAPSkde;
            serial = false;
            naps_kde = naps_kde.stitch(sample, serial);

            twallSEKDE = toc;
            tcpuSEKDE = cputime-twallSEKDE;


            % extract relevant parameters from object after stich() method
            fail_code = naps_kde.failed;
            x_naps_kde = naps_kde.sx;
            SE_pdf_kde = naps_kde.sPDF;
            rndom.Ns = naps_kde.N;
            binrndom.Ns =  naps_kde.binN;



            toc
            %==========================================================
            % % % % % % % % end of estimate % % % % % % % % %
            %==========================================================

            %%%%%%%%%%%%%%%% calculate LG multipliers %%%%%%%%%%%%%%%%%%%%%
            lagrange_naps_parallel(k,i,j) = sum(max_LG);

            all_LG = naps.LG_vals;
            vals = 0;
            nBlock = size(all_LG,2);
            for lg_idx =1:nBlock

                current_LG = size(all_LG{lg_idx},1);
                vals = vals + current_LG;

            end
            avg_LG = vals / nBlock;

            avg_lagrange_naps_parallel(k,i,j) = avg_LG;

            lagrange_naps_serial(k,i,j) = sum(max_LG);
            lagrange_nmem(k,i,j) = sum(length(lagrange_multipler));

            %%%%%%%%%%%%%%%%%%%%% store meta data %%%%%%%%%%%%%%%%%%%%%%%%%

            % read in actual distribution
            rndom.dist_list();

            % Create final answer file
            rndom.filename = sprintf(['D_', char(rndom.dist_name), ...
                '_T_','%d', '_S_','%d'],i, rndom.Ns);

            % store time of computation
            cpu_vec_se_parallel(k,i) = tcpuSE_parallel;
            wall_vec_se_parallel(k,i) = twallSE_parallel;

            cpu_vec_se_serial(k,i) = tcpuSE_serial;
            wall_vec_se_serial(k,i) = twallSE_serial;

            cpu_vec_nmem(k,i) = tcpuNMEM;
            wall_vec_nmem(k,i) = twallNMEM;

            cpu_kde(k,i) = tcpuKDE;
            wall_kde(k,i) = twallKDE;

            cpu_se_kde(k,i) = tcpuSEKDE;
            wall_se_kde(k,i) = twallSEKDE;


            disp([char(actual.dist_name),...
                ', Trial: ',num2str(i),'/', num2str(trials), ...
                ' sample size: ',num2str(sample_vec(k))])

            naps_pdf_data{1,i} = sample;
            naps_pdf_data{2,i} = SE_x;
            naps_pdf_data{3,i} = SE_pdf;

            nmem_pdf_data{1,i} = sample;
            nmem_pdf_data{2,i} = x_NMEM;
            nmem_pdf_data{3,i} = pdf_NMEM;

            kde_pdf_data{1,i} = sample;
            kde_pdf_data{2,i} = x_KDE;
            kde_pdf_data{3,i} = pdf_KDE;
            
            se_kde_pdf_data{1,i} = sample;
            se_kde_pdf_data{2,i} = x_naps_kde;
            se_kde_pdf_data{3,i} = SE_pdf_kde;

        end

        filename = [char(actual.dist_name),...
            '_t_', num2str(trials), ...
            '_s_',num2str(sample_vec(k))];

        % add -v7.3 to save mat files larger than 2GB
        save(fullfile(write_dir,['naps_', filename, '.mat']), 'naps_pdf_data','-v7.3')
        save(fullfile(write_dir,['nmem_', filename, '.mat']), 'nmem_pdf_data','-v7.3')
        save(fullfile(write_dir,['kde_', filename, '.mat']), 'kde_pdf_data','-v7.3')
        save(fullfile(write_dir,['naps_kde_', filename, '.mat']), 'se_kde_pdf_data','-v7.3')
    end

    % cpu time
    stdCPUtimeSE = horzcat(sample_vec',std(cpu_vec_se_parallel,[],2));
    stdCPUtimeNMEM = horzcat(sample_vec',std(cpu_vec_nmem,[],2));

    avgCPUtimeSE = horzcat(sample_vec',mean(cpu_vec_se_parallel,2));
    avgCPUtimeNMEM = horzcat(sample_vec',mean(cpu_vec_nmem,2));

    cpu.stdTimeSE{j} = horzcat(sample_vec',std(cpu_vec_se_parallel,[],2));
    cpu.stdTimeNMEM{j} = horzcat(sample_vec',std(cpu_vec_nmem,[],2));

    cpu.timeSE{j} = horzcat(sample_vec',mean(cpu_vec_se_parallel,2));
    cpu.timeNMEM{j} = horzcat(sample_vec',mean(cpu_vec_nmem,2));

    cpu_naps = cpu_vec_se_parallel;
    cpu_nmem = cpu_vec_nmem;

    %%%%%%%%%%%%%%%%%%%%%%% build data table %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % CPU cpu_vec_se_serial
    temp = vertcat(utils.reshape_groups(sample_vec',cpu_vec_se_parallel),...
        utils.reshape_groups(sample_vec',cpu_vec_se_serial),...
        utils.reshape_groups(sample_vec',cpu_vec_nmem),...
        utils.reshape_groups(sample_vec',cpu_kde),...
        utils.reshape_groups(sample_vec',cpu_se_kde));

    sample_power = temp(:,1);
    cpu_time = temp(:,2);

    % WALL TIME cpu_vec_se_serial
    temp = vertcat(utils.reshape_groups(sample_vec',wall_vec_se_parallel),...
        utils.reshape_groups(sample_vec',wall_vec_se_serial),...
        utils.reshape_groups(sample_vec',wall_vec_nmem),...
        utils.reshape_groups(sample_vec',wall_kde),...
        utils.reshape_groups(sample_vec',wall_se_kde));

    wall_time = temp(:,2);

    % Distributions
    distribution = repelem(distribution_vector(j), length(temp(:,2)))';
    name = repelem(names(j), length(temp(:,2)))';

    naps_label_parallel = repelem(["NAPS_{parallel}"], size(utils.reshape_groups(sample_vec',cpu_vec_se_parallel), 1));
    naps_label_serial = repelem(["NAPS_{serial}"], size(utils.reshape_groups(sample_vec',cpu_vec_se_serial), 1));
    nmem_label = repelem(["NMEM"], size(utils.reshape_groups(sample_vec',cpu_vec_nmem), 1));
    kde_label = repelem(["KDE"], size(utils.reshape_groups(sample_vec',cpu_kde), 1));
    sekde_label = repelem(["NAPS(KDE)"], size(utils.reshape_groups(sample_vec',cpu_se_kde), 1));

    % Failed
    temp = vertcat(utils.reshape_groups(sample_vec',fail_naps_parallel(:,:,j)),...
        utils.reshape_groups(sample_vec',fail_naps_serial(:,:,j)),...
        utils.reshape_groups(sample_vec',fail_nmem(:,:,j)),...
        utils.reshape_groups(sample_vec',fail_nmem(:,:,j)),... % padding rows with random data for NAPS (KDE) and KDE
        utils.reshape_groups(sample_vec',fail_nmem(:,:,j)));

    fail = temp(:,2);

    % Lagragian
    temp = vertcat(utils.reshape_groups(sample_vec',lagrange_naps_parallel(:,:,j)),...
        utils.reshape_groups(sample_vec',lagrange_naps_serial(:,:,j)),...
        utils.reshape_groups(sample_vec',lagrange_nmem(:,:,j)),...
        utils.reshape_groups(sample_vec',lagrange_nmem(:,:,j)),... % padding rows with random data for NAPS (KDE) and KDE
        utils.reshape_groups(sample_vec',lagrange_nmem(:,:,j)));
    lagrange = temp(:,2);

    estimator = vertcat(naps_label_parallel',naps_label_serial', nmem_label', kde_label', sekde_label');

    % padding NMEM with NaNs because there are no scale/size infromation
    padding = utils.reshape_groups(sample_vec',NaN(length(sample_vec),trials));

    % average Lagragian for all blocks, group in lagrange for nmem
    temp = vertcat(utils.reshape_groups(sample_vec',avg_lagrange_naps_parallel(:,:,j)),...
        padding,...
        padding,...% padding rows with random data for NAPS (KDE) and KDE
        padding,...
        utils.reshape_groups(sample_vec',lagrange_nmem(:,:,j)));
    avg_lagrange = temp(:,2);

    % block size ------------------------

    % max
    bs_max_mat = squeeze(block_size(1,:,:))';
    bs_max = utils.reshape_groups(sample_vec',bs_max_mat);
    max_size = vertcat(bs_max, padding, padding, padding, padding);

    sample_power = max_size(:,1);
    max_size = max_size(:,2);


    % min
    bs_min_mat = squeeze(block_size(2,:,:))';
    bs_min = utils.reshape_groups(sample_vec',bs_min_mat);
    min_size = vertcat(bs_min, padding, padding, padding, padding);
    min_size = min_size(:,2);


    % mean
    bs_mean_mat = squeeze(block_size(3,:,:))';
    bs_mean = utils.reshape_groups(sample_vec',bs_mean_mat);
    mean_size = vertcat(bs_mean, padding, padding, padding, padding);
    mean_size = mean_size(:,2);


    % median
    bs_med_mat = squeeze(block_size(4,:,:))';
    bs_med = utils.reshape_groups(sample_vec',bs_med_mat);
    median_size = vertcat(bs_med, padding, padding, padding, padding);
    median_size = median_size(:,2);


    %std dev
    bs_stdev_mat = squeeze(block_size(4,:,:))';
    bs_stdev = utils.reshape_groups(sample_vec',bs_stdev_mat);
    std_size = vertcat(bs_stdev, padding, padding, padding, padding);
    std_size = std_size(:,2);

    blocksize = table(max_size, min_size, mean_size, median_size, std_size);


    % block scale ------------------------

    % max
    bs_max_mat = squeeze(block_scale(1,:,:))';
    bs_max = utils.reshape_groups(sample_vec',bs_max_mat);
    max_scale = vertcat(bs_max, padding, padding, padding, padding);
    max_scale = max_scale(:,2);

    % min
    bs_min_mat = squeeze(block_scale(2,:,:))';
    bs_min = utils.reshape_groups(sample_vec',bs_min_mat);
    min_scale = vertcat(bs_min, padding, padding, padding, padding);
    min_scale = min_scale(:,2);

    % mean
    bs_mean_mat = squeeze(block_scale(3,:,:))';
    bs_mean = utils.reshape_groups(sample_vec',bs_mean_mat);
    mean_scale = vertcat(bs_mean, padding, padding, padding, padding);
    mean_scale = mean_scale(:,2);

    % median
    bs_med_mat = squeeze(block_scale(4,:,:))';
    bs_med = utils.reshape_groups(sample_vec',bs_med_mat);
    median_scale = vertcat(bs_med, padding, padding, padding, padding);
    median_scale = median_scale(:,2);

    %std dev
    bs_stdev_mat = squeeze(block_size(4,:,:))';
    bs_stdev = utils.reshape_groups(sample_vec',bs_stdev_mat);
    std_scale = vertcat(bs_stdev, padding, padding, padding, padding);
    std_scale = std_scale(:,2);

    blockscale = table(max_scale, min_scale, mean_scale, median_scale, std_scale);

    %     dist_table = table(distribution, name, estimator, sample_power, cpu_time, fail, lagrange, blocksize, blockscale);
    dist_table = table(distribution, name, estimator, sample_power, wall_time, cpu_time, fail, lagrange, avg_lagrange, blocksize, blockscale);
    dist_table = splitvars(dist_table);

    % append table per distribution to global table containing data for all
    % distributions
    global_table = [global_table; dist_table];

end

writetable(global_table,fullfile(parent_dir,table_name))



end