classdef analysis
    methods(Static)
        function [htx, hty, htx_act, hty_act] = heavy(xcdf, cdf, obj, xscale)

            % find all x coordinates > 0
            ind = find(xcdf>0);

            % evaluate heavy tails
            obj.x = xcdf(ind);
            obj = dist_list(obj);
            if isequal(lower(xscale), 'log')
                htx = log10(xcdf(ind));
                htx_act = log10(obj.x);
            else
                htx = xcdf(ind);
                htx_act = obj.x;
            end

            hty = log10(1-cdf(ind));
            hty_act = log10(1-obj.cdf_y);
        end

        function val = mpe(N,P,Q)
            
            % number of data points in sample
            m = length(Q);
            % N is sample size used to create estimate
            epsilon = 0.01/N;
            % percent error
            err = 100*abs(P - Q)./max(Q,epsilon);
            % mean percent error
            val = sum(err)/m;
        end

        function val = mse(fact, fs)
            n = length(fs);
            val = (1/n)*sum((fact-fs).^2);

        end

        function [metric_dist, quantiles, xq, fq] = quant_metric(N,xs, fs, Fs, qnum, d_name, metric)

            actual = distributions;
            actual.generate_data = false;

            q_min = 0.05;
            q_max = 0.95;

            quantiles = linspace(q_min, q_max, qnum)';

            [Fs, a1, ~] = unique(Fs);
            xs = xs(a1);
            fs = fs(a1);

            xq = interp1(Fs,xs,quantiles);
            fq = interp1(xs,fs,xq);

            % get actual distribution
            actual.min_limit = min(xs);
            actual.max_limit = max(xs);
            actual.dist_name = d_name;

            metric_dist = zeros(qnum-1,1);
            for q = 2:qnum

                trunc_xs = linspace(xq(q-1), xq(q), 1000);
                trunc_fs = interp1(xs,fs,trunc_xs);

                actual.x = linspace(min(trunc_xs),max(trunc_xs),length(trunc_xs));
                actual = actual.dist_list();
                fact = actual.pdf_y;

                if sum(~isfinite(fact)) + sum(~isfinite(trunc_fs))
                    sum(~isfinite(fact))
                    sum(~isfinite(trunc_fs))
                    error('the inputs contain non-finite values!')
                end

                if isequal(upper(metric), 'MSE')
                    metric_dist(q-1) = analysis.mse(fact, trunc_fs);
                elseif isequal(upper(metric), 'MPE')
                    metric_dist(q-1) = analysis.mpe(N,fact, trunc_fs);
                end
            end
        end

        function Fs = trapz(xs,fs)

            % length of sample estimate
            ns = length(xs);
            % numerically integrate PDF
            Fs = zeros(ns,1);
            for n = 2:ns
                Fs(n) = Fs(n-1)+ (xs(n)-xs(n-1))*(fs(n) + fs(n-1))/2;
            end

        end

    end
end