classdef blocks < NAPS % inherit NAP properties i.e. p vector and max block size
    properties
        sample;
        dx;
        dxs;
        Ns;
        window;
        binMin;
        binNs;
    end
    methods(Static)
        function [j,nBlocks,kBlockLower,kBlockUpper,kList,T,Rlevel,R0] = ...
                bin_width_size(obj)

            obj.Ns = length(obj.sample);

            if( obj.Ns < obj.binNs )
                nBlocks = 1;
            else
                % determine number of blocks and their sizes
                [kList,T,Rlevel,R0] = obj.r_tree(obj);
                % remove tiny left and right
                if( kList(1) < obj.binNs )
                    kList = kList(2:end);
                end
                if( obj.Ns - kList(end) + 1 < obj.binNs )
                    kList = kList(1:end-1);
                end
                j = length(kList);
                nBlocks = 2*j + 1;
                kBlockLower = zeros(1,nBlocks);
                kBlockUpper = zeros(1,nBlocks);
            end
        end

        function [pList,T,Rlevel,R0] = r_tree(obj)

            % create T threshold-------------------------------------------
            T = 0.5255 * obj.Ns ^ 1.1;
            obj.window = min(10, obj.Ns - 1);

            maxLevel = obj.max_lvl;

            % minimum blocksize--------------------------------------------
            obj.binMin = 10;
            if obj.binMin > obj.Ns
                error('sample size too small for window size')
            end

            % track number of branches per level
            nbranch = 1;
            % set end points of sample length as partition left (pL) and
            % partion right (pR)
            pL = 1;
            pR = obj.Ns;
            % initialize vector to track all created partitions
            pList = [pL pR];
            % track every attempted partition for all levels
            plevel = {{[1;obj.Ns]}};
            % initialize array to track newly created partitions
            % for plotting purposes
            pdiff = {[1;obj.Ns]};
            % calcualte inital R of intire sample
            R0 = obj.get_ratio(obj, obj.sample);

            % clear array to hold all BR values per block per level
            Rlevel = {R0};
            % beggin level loop
            if (R0 / T) > 1
                for jj = 1:maxLevel
                    % vector to hold all attempted partitions per level
                    plevHold = [];
                    % vector to hold all BR values per level
                    RHold = [];
                    % beggin branch loop
                    for b = 1:nbranch
                        % define block size (B)
                        B = pList(b+1) - pList(b);
                        T = 0.5255 * B ^ 1.1;
                        if obj.binMin >= B
                            continue
                        end
                        % update left
                        boundryL = obj.binMin + 1;
                        boundryR = B - obj.binMin;
                        % block to small for for minimization
                        % given window size
                        if boundryR - boundryL < 3
                            break;
                        end

                        % golden ration bifraction minimization
                        [rL,rR,partition] = obj.minimize_r(obj, obj.sample(pList(b):pList(b+1)));
                        
                        % find minimum BR for two newly created blocks
                        Rmin = min(rL,rR);
                        Rmax = max(rL,rR);
                        R = (Rmin + Rmax) / 2;
                        % update block boundaries of sample with correctly
                        % placed partition
                        newPar = pList(b) + partition;
                        % hold all BRs per level for later evaluation
                        RHold = [RHold,R];
                        % un-balanced tree-----------------------------
                        %                         if BR <= T
        
                        if (R / T) > 1 %BR >= T
                            plevHold = [plevHold, newPar];
                        end
                    end
                    pList = sort([pList plevHold]);
                    % update nbranch
                    nbranch = length(pList) - 1;
                    % assign partition list to array for plotting
                    plevel{jj+1,1} = {pList'};
                    % exit for special cases where B < binMin
                    % or bRight - bLeft < 3
                    if isempty(RHold)
                        break;
                    end
                    % assign R per level to array for plotting
                    Rlevel{jj+1,1} = RHold;
                    % find newly accepted partitions
                    [C,~] = setdiff(plevel{jj+1,1}{1,1}(:,1),...
                        plevel{jj,1}{1,1}(:,1));
                    % STOP SEARCH: if no new partitions are accepted
                    if isempty(C)
                        break;
                    end
                    % update changes with newly created partitions
                    pdiff{jj+1,1} = C;
                end
                pList = pList';

                % SPLITTING ROUTINE FOR LARGE SAMPLES =====================
                % vector to to add new partitons too
                largeNcheck = pList;
                % vector to hold updated partition list
                holder = pList;
                % while loop flag
                runSplit = true;

                while runSplit
                    % triggers exit flag for while loop
                    splitCount = 0;
                    % loop over modified partition list (holder)
                    for k = 1:length(holder)-1
                        % add partion between elements when diff > max_bs
                        if holder(k+1)-holder(k) > obj.max_bs
                            split = floor((holder(k+1)-holder(k))/2);
                            % update new partiton list
                            largeNcheck = [largeNcheck; largeNcheck(k)+ split];
                            % update counter: number of found splits
                            splitCount = splitCount + 1;
                        end
                    end
                    largeNcheck = sort(largeNcheck);
                    holder = largeNcheck;
                    % if no splits exit routine
                    if splitCount == 0
                        runSplit = false;
                    end
                end
                pList = largeNcheck;
            end

        end

        function [rL,rR,partition] = minimize_r(obj, sample)

            Ns = length(sample);
            bLeft = 1 + obj.binMin;
            bRight = Ns - obj.binMin;
            goldenR = (1+sqrt(5))/2;
            dxr = [];
            loopCount = 0;
            partition = ceil(Ns/2);

            while bRight-bLeft > 2

                % define partitions ---------------------------------------
                bLeft1 = partition;
                bRight1 = Ns - partition;
                rRight1 = obj.get_ratio(obj, sample(partition:end));
                rLeft1 = obj.get_ratio(obj, sample(1:partition));
                dxrC = abs(rLeft1-rRight1);
                % --------------------
                leftPar = partition - 1;
                bLeft2 = leftPar;
                bRight2 = Ns - leftPar ;
                rRight2 = obj.get_ratio(obj, sample(leftPar:end));
                rLeft2 = obj.get_ratio(obj, sample(1:leftPar));
                dxrL = abs(rLeft2-rRight2);
                % --------------------
                rightPar = partition + 1;
                bLeft3 = rightPar;
                bRight3 = Ns - rightPar;
                rRight3 = obj.get_ratio(obj, sample(rightPar:end));
                rLeft3 = obj.get_ratio(obj, sample(1:rightPar));
                dxrR = abs(rLeft3-rRight3);
                % --------------------
                dxCR = dxrC - dxrR;
                dxLC = dxrL - dxrC;
                dxLR = dxrL - dxrR;

                % parition update conditions ------------------------------
                % dxrR is smallest
                if dxCR >= 0 && dxLR > 0
                    bLeft = leftPar;
                    if bRight-bLeft < 2
                        dxr = horzcat(dxr,[dxrC;bLeft1;rLeft1;bRight1;rRight1]);
                    else
                        % Shrink --->
                        dxr = horzcat(dxr,[dxrR;bLeft3;rLeft3;bRight3;rRight3]);
                        partition = round((bLeft+bRight*goldenR)/(1+goldenR));
                    end
                end
                % dxrL is smallest
                if dxLC <= 0 && dxLR < 0
                    bRight = rightPar;
                    if bRight-bLeft < 2
                        dxr = horzcat(dxr,[dxrC;bLeft1;rLeft1;bRight1;rRight1]);
                    else
                        % <--- Shrink
                        dxr = horzcat(dxr,[dxrL;bLeft2;rLeft2;bRight2;rRight2]);
                        partition = round((bLeft*goldenR+bRight)/(1+goldenR));
                    end
                end
                % dxrC is smallest
                if dxLC > 0 && dxCR < 0
                    dxr = horzcat(dxr,[dxrC;bLeft1;rLeft1;bRight1;rRight1]);
                    break
                end
                % dxrC = dxrL = dxrR = 0
                if dxLC == 0 || dxCR == 0 || dxLR == 0
                    dxr = horzcat(dxr,[dxrC;bLeft1;rLeft1;bRight1;rRight1]);
                    break
                end
                loopCount = loopCount + 1;
            end

            dxr = dxr';
            rL = dxr(end,3);
            rR = dxr(end,5);
            partition = dxr(end,2);
        end

        function r = get_ratio(obj, sample)

            % TODO: debug mapping. Errors seem to occur for large samples.
%             % create boolean mask for subsample from entire sample
%             mask = (min(sample)<=obj.sample & max(sample)>=obj.sample);
%             % change one binary 1 value to 0 to create mask for difference array
%             % indexes where mask is true i.e. 1
%             mask_idx= find(mask==1);
%             % change last true value on the right to false
%             mask(mask_idx(end)) = 0;
%             mask = mask(1:end-1);
%             % get the dx values for this subsample in sorted order
%             sort_mask = ismember(obj.dxs, obj.dx(mask)');
%             dx = obj.dxs(sort_mask);
% 
%             dxMin = mean(dx(1:obj.window));
%             dxMax = mean(dx(end-obj.window+1:end));

            n = length(sample);
            dx = zeros(1,n-1);
            dx(1:n-1) = sample(2:n) - sample(1:n-1);
            % REQUIRED to sort the magnitude of differences
            dx = sort(dx');
            dxMin = mean(dx(1:obj.window));
            dxMax = mean(dx(end-obj.window+1:end));
            
            % -------------------------------------------------------------
            % r -> inf, dxMin << dxMax and r -> 1, dxMin ~ dxMax
            % -------------------------------------------------------------
            r = dxMax/dxMin;

        end

       
    end
end

