
close all;

addpath("functions/")
addpath("functions_plotting/")

publicationQuality();
save_figs = true;

dir_name = fullfile('figures','run_times');
status = mkdir(dir_name);

% import data table
filename = fullfile('data_kde', 'meta_data_100.dat');
data = readtable(filename);

% define labels for figures
names = ["Uniform", "Normal","Beta(0.5,1.5)","Beta(2,0.5)","Beta(0.5,0.5)","Generalized-Pareto", "Stable"];

% filter data into relavent tables
data_naps_parallel = data('NAPS_{parallel}'==convertCharsToStrings(data.estimator),:);
data_naps_serial = data('NAPS_{serial}'==convertCharsToStrings(data.estimator),:);
data_nmem = data('NMEM'==convertCharsToStrings(data.estimator),:);
data_naps_kde = data('NAPS(KDE)'==convertCharsToStrings(data.estimator),:);
data_kde= data('KDE'==convertCharsToStrings(data.estimator),:);

% CPU and Wall time =======================================================

xlimits = [7,23];
ylimits = [-4,6];

fig_name = 'line_NAP_{parallel} Wall Time per Distribution';
figure('Name',fig_name)
% compute average
data_naps_parallel = convertvars(data_naps_parallel,["distribution","name","estimator","sample_power"],"categorical");
mean_data__parallel = groupsummary(data_naps_parallel,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);
T = mean_data__parallel;
%group based on first column - gender column
% modify 1 to appropriate column number
G = findgroups(T{:, 1});
% split table based on first column - gender column
T_split = splitapply( @(varargin) varargin, T , G);
% allocate empty cell array fo sizxe equal to number of rows in T_Split
subTables = cell(size(T_split, 1));
% create sub tables
for i = 1:size(T_split, 1)
subTables{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables)
    data_temp = subTables{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_wall_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-','DisplayName',string(data_temp.name(1)))
end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'line_NMEM Wall Time per Distribution';
figure('Name',fig_name)
% compute average
data_nmem = convertvars(data_nmem,["distribution","name","estimator","sample_power"],"categorical");
mean_data_nmem = groupsummary(data_nmem,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);

T = mean_data_nmem;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables)
    data_temp = subTables{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_wall_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-', 'DisplayName',string(data_temp.name(1)))

end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'line_NAP_{parallel} CPU per Distribution';
figure('Name',fig_name)
% compute average
data_naps_parallel = convertvars(data_naps_parallel,["distribution","name","estimator","sample_power"],"categorical");
mean_data_parallel = groupsummary(data_naps_parallel,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);

T = mean_data_parallel;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables_naps = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables_naps{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables_naps)
    data_temp = subTables_naps{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_cpu_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-','DisplayName',string(data_temp.name(1)))
end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'line_NMEM CPU per Distribution';
figure('Name',fig_name)
% compute average
data_nmem = convertvars(data_nmem,["distribution","name","estimator","sample_power"],"categorical");
mean_data_nmem = groupsummary(data_nmem,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);

T = mean_data_nmem;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables)
    data_temp = subTables{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_cpu_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-', 'DisplayName',string(data_temp.name(1)))

end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'line_NAP(KDE) CPU per Distribution';
figure('Name',fig_name)
% compute average
data_naps_kde = convertvars(data_naps_kde,["distribution","name","estimator","sample_power"],"categorical");
mean_data_naps_kde = groupsummary(data_naps_kde,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);

T = mean_data_naps_kde;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables_naps = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables_naps{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables_naps)
    data_temp = subTables_naps{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_cpu_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-','DisplayName',string(data_temp.name(1)))
end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'line_KDE CPU per Distribution';
figure('Name',fig_name)
% compute average
data_kde = convertvars(data_kde,["distribution","name","estimator","sample_power"],"categorical");
mean_data_kde = groupsummary(data_kde,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);

T = mean_data_kde;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables)
    data_temp = subTables{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_cpu_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-', 'DisplayName',string(data_temp.name(1)))

end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'line_NAP(KDE) wall time per Distribution';
figure('Name',fig_name)
% compute average
data_naps_kde = convertvars(data_naps_kde,["distribution","name","estimator","sample_power"],"categorical");
mean_data_naps_kde = groupsummary(data_naps_kde,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);

T = mean_data_naps_kde;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables_naps = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables_naps{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables_naps)
    data_temp = subTables_naps{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_wall_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-','DisplayName',string(data_temp.name(1)))
end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'line_KDE wall time per Distribution';
figure('Name',fig_name)
% compute average
data_kde = convertvars(data_kde,["distribution","name","estimator","sample_power"],"categorical");
mean_data_kde = groupsummary(data_kde,["distribution","name","estimator","sample_power"],"mean",["wall_time","cpu_time"]);

T = mean_data_kde;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

hold on;
for i = 1:length(subTables)
    data_temp = subTables{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_wall_time);    

    x = log(x)/log(2);
    y = log10(y);

    plot(x,y,'.-', 'DisplayName',string(data_temp.name(1)))

end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(CPU) Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

% Compare wall time NAPS with KDE =========================================
data_filtered = data('NAPS_{serial}'~=convertCharsToStrings(data.estimator),:);
data_filtered = data_filtered('NMEM'~=convertCharsToStrings(data_filtered.estimator),:);

% remove wall time for samples sizes smaller than 512
[G1,~,~]=unique(data_filtered(:,2));
[G2,~,~]=unique(data_filtered(:,3));
new_table = table();
for g1_idx = 1:size(G1)
    for g2_idx = 1:size(G2)
        temp = data_filtered(strcmp(data_filtered.name, G1.name{g1_idx}) & strcmp(data_filtered.estimator, G2.estimator{g2_idx}) & 512 == data_filtered.sample_power, :);
        temp2 = data_filtered(strcmp(data_filtered.name, G1.name{g1_idx}) & strcmp(data_filtered.estimator, G2.estimator{g2_idx}) & 512 < data_filtered.sample_power, :);
        temp(1,:) = [];
        new_table = [new_table; temp; temp2];
    end
end

% calcuate log(wall time)
func = @(x) log10(x);
new_table(:,5) = varfun(func,new_table(:,5));

fig_name = 'line_KDE wall time per Distribution';
figure('Name',fig_name)
% compute average and standard deviation
data_total = convertvars(new_table,["estimator","sample_power"],"categorical");
mean_data_total = groupsummary(data_total,["estimator","sample_power"],["mean","std"],["wall_time","cpu_time"]);

T = mean_data_total;
G = findgroups(T{:, 1});
T_split = splitapply( @(varargin) varargin, T , G);
subTables = cell(size(T_split, 1));
for i = 1:size(T_split, 1)
subTables{i} = table(T_split{i, :}, 'VariableNames', ...
T.Properties.VariableNames);
end

cc = ["#00FF00","#7E2F8E",'r'];
hold on;
for i = 1:length(subTables)
    data_temp = subTables{i};
    x = double(string(data_temp.sample_power));
    y = double(data_temp.mean_wall_time);
    err = data_temp.std_wall_time;

    x = log(x)/log(2);
    err = log10(err);

    h = errorbar(x,y,err,'DisplayName',string(data_temp.estimator(1)),'color',cc(i));
end
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('log$_{2}(N)$','Interpreter','latex')
ylabel('log$_{10}$(sec) Wall Time','Interpreter','latex')
legend('Location','northwest')
xlim(xlimits)
ylim(ylimits)
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
    saveas(bp, fullfile(dir_name, [fig_name, '.fig']))
end

