
close all;

addpath("functions/")
addpath("functions_plotting/")

publicationQuality();
save_figs = true;

dir_name = fullfile('figures_manuscript','run_times_2');
status = mkdir(dir_name);

% import data table
filename = fullfile('data_cpu_40_wall', 'meta_data_100.dat');
filename = fullfile('data_kde', 'meta_data_100.dat');
data = readtable(filename);

% define labels for figures
names = ["Tri-Modal-Normal","Uniform","Normal","Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5","Generalized-Pareto","Stable"];

% filter data into relavent tables
data_naps_parallel = data('NAPS_{parallel}'==convertCharsToStrings(data.estimator),:);
data_naps_serial = data('NAPS_{serial}'==convertCharsToStrings(data.estimator),:);
data_nmem = data('NMEM'==convertCharsToStrings(data.estimator),:);

fig_name = 'NAP_{parallel} Avg lagrange multiplier';
figure('Name',fig_name)
b = boxchart(log(data_naps_parallel.sample_power)/log(2), data_naps_parallel.avg_lagrange, 'GroupByColor',data_naps_parallel.distribution);
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('$log_{2}(N)$','Interpreter','latex')
ylabel('Avg lagrange','Interpreter','latex')
legend('Location','northwest')
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'NMEM Avg lagrange multiplier';
figure('Name',fig_name)
b = boxchart(log(data_nmem.sample_power)/log(2), data_nmem.avg_lagrange, 'GroupByColor',data_nmem.distribution);
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('$log_{2}(N)$','Interpreter','latex')
ylabel('Avg lagrange','Interpreter','latex')
legend('Location','northwest')
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'NAP Max lagrange multiplier';
figure('Name',fig_name)
b = boxchart(log(data_naps_parallel.sample_power)/log(2), data_naps_parallel.lagrange, 'GroupByColor',data_naps_parallel.distribution);
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('$log_{2}(N)$','Interpreter','latex')
ylabel('Max Lagrange Multiplier','Interpreter','latex')
legend('Location','northwest')
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

fig_name = 'NMEM Max lagrange multiplier';
figure('Name',fig_name)
b = boxchart(log(data_nmem.sample_power)/log(2), data_nmem.lagrange, 'GroupByColor',data_nmem.distribution);
bp = gca;
bp.XAxis.TickLabelInterpreter = 'latex';
xlabel('$log_{2}(N)$','Interpreter','latex')
ylabel('Max Lagrange Multiplier','Interpreter','latex')
legend('Location','northwest')
if save_figs
    saveas(bp, fullfile(dir_name, [fig_name, '.png']))
end

