close all;

addpath("functions/")
addpath("functions_plotting/")

publicationQuality();

plot_mse_dist = false;
save_figs = true;

table_name = 'mse_kl_100.dat';
fig_dir = fullfile('figures','kl_mpe_naps_v_nmem');
% fig_dir = fullfile('figures','kl_mpe_naps_v_kde');
write_dir = fullfile('data_kde','kl_mse_cpu_40_t_100');

status = mkdir(write_dir);
status = mkdir(fig_dir);

data = readtable(fullfile(write_dir,table_name));

data.estimator(strcmp(data.estimator,'NAP')) = {'NAPS'};
data.kl_total(isinf(data.kl_total)) = NaN;

% NAPS vs NMEM
data = data('NAPS(KDE)'~=convertCharsToStrings(data.estimator),:);
data = data('NAPS(KDE)_{norm}'~=convertCharsToStrings(data.estimator),:);
data = data('NAPS(NMEM)'~=convertCharsToStrings(data.estimator),:);
data.estimator(strcmp(data.estimator,'NAPS(NMEM)_norm')) = {'NAPS'};

% NAPS(NMEM) vs NAPS(KDE)
% data = data('NAPS(KDE)'~=convertCharsToStrings(data.estimator),:);
% data = data('NMEM'~=convertCharsToStrings(data.estimator),:);
% data = data('NAPS(NMEM)'~=convertCharsToStrings(data.estimator),:);
% data.estimator(strcmp(data.estimator,'NAPS(NMEM)_norm')) = {'NAPS(NMEM)'};
% data.estimator(strcmp(data.estimator,'NAPS(KDE)_{norm}')) = {'NAPS(KDE)'};

data = data('NAPS_{serial}'~=convertCharsToStrings(data.estimator),:);
% data = data('NMEM'~=convertCharsToStrings(data.estimator),:);
data = data('KDE'~=convertCharsToStrings(data.estimator),:);

data_nap = data('NAPS(NMEM)'==convertCharsToStrings(data.estimator),:);
data_naps_kde = data('NAPS(KDE)'==convertCharsToStrings(data.estimator),:);
data_kde = data('KDE'==convertCharsToStrings(data.estimator),:);

d_vec = ["Trimodal-Normal","Uniform","Normal","Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5","Generalized-Pareto","Stable"];
names = ["Trimodal-Normal","Uniform", "Normal", "Beta(0.5,1.5)", "Beta(2,0.5)", "Beta(0.5,0.5)","Generalized-Pareto","Stable"];

%%%%%%%%%%%%%%%%%%%%%%%%%% Plot MPE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idx = 1:length(names)

    tester = data.name;
    mask = data.name == names(idx);
    data_dist = data(mask,:);
    %     data_dist.kl_total = log10(data_dist.kl_total);

    fig_name = sprintf('mpe_%s',d_vec(idx));
%         fig_name = sprintf('kde_mpe_%s',d_vec(idx));
    f = figure('Name',fig_name);
    boxchart(log(data_dist.sample_power)/log(2), log10(data_dist.mpe_total), 'GroupByColor',data_dist.estimator);
    bp = gca;
    bp.XAxis.TickLabelInterpreter = 'latex';
    xlabel('log$_{2}(N)$','Interpreter','latex', 'FontName', 'Liberation Serif')
    ylabel('log$_{10}(MPE)$','Interpreter','latex', 'FontName', 'Liberation Serif')
    legend('Location','northeast', 'FontName', 'Liberation Serif')
    xlim([8,23])
    title(convertCharsToStrings(names(idx)), 'FontName', 'Liberation Serif')

    %     Change the boxplot color
    boxes = get(gca, 'Children');
    box1 = boxes(1);
    box2 = boxes(2);
    set(box1, 'BoxFaceColor', 'b');
    set(box2, 'BoxFaceColor', 'r');
    set(box1, 'MarkerColor', 'b');
    set(box2, 'MarkerColor','r');

%         boxes = get(gca, 'Children');
%         box1 = boxes(1);
%         box2 = boxes(2);
%         set(box1, 'BoxFaceColor', 'r');
%         set(box2, 'BoxFaceColor', "#7E2F8E");
%         set(box1, 'MarkerColor', 'r');
%         set(box2, 'MarkerColor', "#7E2F8E");

    str = ["Trimodal-Normal","Uniform", "Normal", "Beta(0.5,1.5)", "Beta(2,0.5)","Beta(0.5,0.5)","Generalized-Pareto","Stable"];
    for b = 1:size(str,2)
        if strcmp(names(idx),str(b))
            if strcmp(names(idx),"Stable")
                legend('Location','east')
                break;
            elseif strcmp(names(idx),"Beta(0.5,1.5)")
                ylim([-1.5,inf])
                legend('Location','southwest')
                break;
            else
                legend('Location','southwest')
                break;
            end
        else
            legend('Location','northeast')
        end
    end

%     str = ["Trimodal-Normal","Uniform", "Normal"];
%     for b = 1:size(str,2)
%         if strcmp(names(idx),str(b))
%             if strcmp(names(idx),"None")
%                 legend('Location','east')
%                 break;
%             else
%                 legend('Location','southwest')
%                 break;
%             end
%         else
%             legend('Location','northeast')
%         end
%     end
    if save_figs
        fig_export(f ,fullfile(fig_dir, fig_name),'naps')
    end
end



