close all;

addpath("functions/")
addpath("functions_plotting/")

publicationQuality();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%% To Plot %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plot_mse_dist = false;
table_name = 'mse_kl_set0.dat';

plot_pdf = false;
plot_cdf = false;
plot_sqr = false;
plot_heavy = true;
save_figs = true;

% choose to visualise figures or not
fig_plot = 'on';

%%%%%%%%%%%%%%%%%%%%%%%%%%% Import data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% figure directory
fig_dir = fullfile('figures','q_norm');
% fig_dir = fullfile('figures','heavy_norm');
% fig_dir = fullfile('figures','100_trials_kde_all_dists_2');
status = mkdir(fig_dir);

% Create actual distrobution onjects
actual = distributions;

% Path to directory
data_dir = 'data_kde';
dir_name = fullfile(data_dir);
cpu_n = 40;

% Define the etimates to plot

% Sample range
% n_vec = 2.^[8,9,10,11,12,13,14,15,16,17,18,19,20,21,22];
n_vec = 2.^[10];

% Distribution range
d_vec = ["Trimodal-Normal","Uniform","Normal","Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5","Generalized-Pareto","Stable"];
names = ["Trimodal-Normal","Uniform","Normal","Beta(0.5,1.5)","Beta(2,0.5)", "Beta(0.5,0.5)", "Generalized-Pareto","Stable"];
d_vec = ["Stable"];
names = ["Stable"];
% d_vec = ["Beta-a0p5-b0p5"];
% names = ["Beta(0.5,0.5)"];

% Trials per sample
trials = 100;
% evaluate specific number of trials
plt_trials = 2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Load data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize empty structure to store data in
n = length(n_vec);
d = length(d_vec);
t = length(d_vec);
naps = cell(d,n,t,7);
nmem = cell(d,n,t,7);
naps_kde = cell(d,n,t,7);
kde = cell(d,n,t,7);

for i = 1:length(d_vec)
    for j = 1:length(n_vec)


        % Load data from estimate directory
        load_dir = fullfile(dir_name,sprintf('%s_cpu_%d_t_%d', d_vec(i), cpu_n, trials));
        disp(['Loading: ', load_dir])
        % Load data from estimate directory
        load(fullfile(load_dir,['naps_',num2str(d_vec(i)),'_t_',num2str(trials),'_s_', num2str(n_vec(j)),'.mat']));
        load(fullfile(load_dir,['nmem_',num2str(d_vec(i)),'_t_',num2str(trials),'_s_', num2str(n_vec(j)),'.mat']));
        load(fullfile(load_dir,['kde_',num2str(d_vec(i)),'_t_',num2str(trials),'_s_', num2str(n_vec(j)),'.mat']));
        load(fullfile(load_dir,['naps_kde_',num2str(d_vec(i)),'_t_',num2str(trials),'_s_', num2str(n_vec(j)),'.mat']));


        for k = 1:trials
            %         for k = 1:1

            % distribution names
            naps{i,j,k,1} = d_vec(i);
            nmem{i,j,k,1} = d_vec(i);
            naps_kde{i,j,k,1} = d_vec(i);
            kde{i,j,k,1} = d_vec(i);
            % sample size
            naps{i,j,k,2} = n_vec(j);
            nmem{i,j,k,2} = n_vec(j);
            naps_kde{i,j,k,2} = n_vec(j);
            kde{i,j,k,2} = n_vec(j);

            % Make sure that distribution names are saved as character vectors

            % track calculations
            disp(['dist: ',num2str(i),'/',num2str(length(d_vec)),...
                ' s: ',num2str(j),'/',num2str(length(n_vec)),...
                ' t: ',num2str(k),'/',num2str(trials)])

            % x
            naps_data = naps_pdf_data;
            nmem_data = nmem_pdf_data;
            kde_data = kde_pdf_data;
            naps_kde_data = se_kde_pdf_data;

            naps{i,j,k,3} = naps_data{2,k}'; % make sure column vector
            nmem{i,j,k,3} = nmem_data{2,k};
            naps_kde{i,j,k,3} = naps_kde_data{2,k}'; % make sure column vector
            kde{i,j,k,3} = kde_data{2,k}';

            % pdf
            naps{i,j,k,4} = naps_data{3,k}'; % make sure column vector
            nmem{i,j,k,4} = nmem_data{3,k};
            naps_kde{i,j,k,4} = naps_kde_data{3,k}'; % make sure column vector
            kde{i,j,k,4} = kde_data{3,k}';

            % normalize stitched pdf results (only for un-normalized raw data)
            % integrate

            % NAPS(NMEM)
            area = 0;
            for idx = 1:length(naps{i,j,k,3})-1
                area = area + naps{i,j,k,4}(idx)*(naps{i,j,k,3}(idx+1) - naps{i,j,k,3}(idx));
            end
            naps{i,j,k,4} = naps{i,j,k,4} / area;
            % NAPS(KDE)
            area = 0;
            for idx = 1:length(naps_kde{i,j,k,3})-1
                area = area + naps_kde{i,j,k,4}(idx)*(naps_kde{i,j,k,3}(idx+1) - naps_kde{i,j,k,3}(idx));
            end
            naps_kde{i,j,k,4} = naps_kde{i,j,k,4} / area;


            % cdf
            naps{i,j,k,5} = analysis.trapz(naps{i,j,k,3}, naps{i,j,k,4});
            naps_kde{i,j,k,5} = analysis.trapz(naps_kde{i,j,k,3}, naps_kde{i,j,k,4});

            nmem{i,j,k,5} = analysis.trapz(nmem_data{2,k}, nmem_data{3,k});
            kde{i,j,k,5} = analysis.trapz(kde_data{2,k}, kde_data{3,k});

            % u and SQR
            [naps{i,j,k,6}, naps{i,j,k,7}] = utils.sqr(naps{i,j,k,3}, naps{i,j,k,4},naps_data{1,k});
            [naps_kde{i,j,k,6}, naps_kde{i,j,k,7}] = utils.sqr(naps_kde{i,j,k,3}, naps_kde{i,j,k,4}, naps_kde_data{1,k});

            [nmem{i,j,k,6}, nmem{i,j,k,7}] = utils.sqr(nmem_data{2,k}, nmem_data{3,k}, nmem_data{1,k});
            [kde{i,j,k,6}, kde{i,j,k,7}] = utils.sqr(kde_data{2,k}, kde_data{3,k}, kde_data{1,k});


        end
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%% Ploting data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% MPE Plots %%%

% tracks mpe for all samples
if plot_mse_dist

    % number of quantiles
    qnum = 50;

    % x values for given quantiles
    for d_idx = 1:size(naps, 1)

        mse_dist_nmem = zeros(size(naps, 2),plt_trials);
        mpe_dist_nmem = zeros(size(naps, 2),plt_trials);
        mse_dist_naps = zeros(size(naps, 2),plt_trials);
        mpe_dist_naps = zeros(size(naps, 2),plt_trials);
        for n_idx = 1:size(naps, 2)

            mse_dist_naps_per_q = zeros(qnum-1,plt_trials);
            mse_dist_nmem_per_q = zeros(qnum-1,plt_trials);
            mpe_dist_nmem_per_q = zeros(qnum-1,plt_trials);
            mpe_dist_naps_per_q = zeros(qnum-1,plt_trials);
            quantiles = zeros(qnum,plt_trials);

            % use parfor to speed up calculation at a memory cost
            for t_idx = 1:plt_trials
%             parfor t_idx = 1:plt_trials

                actual = distributions;
                actual.generate_data = false;
                % define distribution name for plotting and building a data table
                actual.dist_name = d_vec(d_idx);

                % track calculations
                disp(['quantile dist: ',num2str(d_idx),'/',num2str(size(naps, 1)),...
                    ' s: ',num2str(n_idx),'/',num2str(size(naps, 2)),...
                    ' t: ',num2str(t_idx),'/',num2str(plt_trials)])

                % NMEM ------------------------------------------------------
                xs = nmem{d_idx,n_idx,t_idx,3}(:,1);
                fs = nmem{d_idx,n_idx,t_idx,4}(:,1);
                Fs = nmem{d_idx,n_idx,t_idx,5}(:,1);
                % get actual distribution
                actual.min_limit = min(xs);
                actual.max_limit = max(xs);
                actual.x = xs;
                actual = actual.dist_list();


                % MPE per quantile
                [mpe_dist_nmem_per_q(:,t_idx), ~,~,~] = analysis.quant_metric(n_vec(n_idx), xs, fs, Fs, qnum, d_vec(d_idx),'MPE');
                % MPE per trial
                % mpe_dist_nmem(n_idx, t_idx) = analysis.mpe(actual.pdf_y, fs');

                % NAP ------------------------------------------------------
                xs = naps{d_idx,n_idx,t_idx,3}(:,1);
                fs = naps{d_idx,n_idx,t_idx,4}(:,1);
                Fs = naps{d_idx,n_idx,t_idx,5}(:,1);

                % get actual distribution
                actual.min_limit = min(xs);
                actual.max_limit = max(xs);
                actual.x = xs;
                actual = actual.dist_list();

                % MPE per quantile
                [mpe_dist_naps_per_q(:,t_idx), quantiles(:,t_idx),xq,fq] = analysis.quant_metric(n_vec(n_idx), xs, fs, Fs, qnum, d_vec(d_idx),'MPE');
                % MPE per trial
                % mpe_dist_naps(n_idx, t_idx) = analysis.mpe(actual.pdf_y, fs');


            end

            % plot quantiles
            fig_name = ['quantile_d_',convertStringsToChars(d_vec(d_idx)),'_s_',num2str(n_vec(n_idx))];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            act_h = plot(xs, Fs, '-k', 'DisplayName', '$F(x)$');
            q_h = plot(xq, quantiles(:,1), 'og', 'DisplayName', sprintf('$Q_{%s}(x)$',[num2str(100/qnum),'\%']));
            pdf_h = plot(xq, fq, '.b', 'DisplayName', '$f(Q^{-1}(y))$');
            xlabel('x')
            ylabel('Q(x)')
            legend([act_h(1),q_h(1),pdf_h(1)],'Interpreter','latex', 'Location','northwest')
            bp = gca;
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end
            
            fig_name = ['avg_mpe_d_',convertStringsToChars(d_vec(d_idx)),'_s_',num2str(n_vec(n_idx))];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            y_naps = mpe_dist_naps_per_q;
            y_nmem = mpe_dist_nmem_per_q;
            x_q = quantiles(1:end-1,1);
            naps_h = plot(x_q, mean(y_naps,2), '-r', 'DisplayName', '$NAPS$');
            nmem_h = plot(x_q, mean(y_nmem,2), '-b', 'DisplayName', '$NMEM$');
            xlabel('Quantile')
            ylabel('\langle MPE \rangle')
            legend([naps_h(1),nmem_h(1)],'Interpreter','latex')
            bp = gca;
            %             bp.YAxis.Scale ="log";
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end

        end
    end
end


%%% PDFs %%%
if plot_pdf
    % Plot data the order of indices corresponds to distribution, sample size,
    % trial. Plot all trials per sample per distrobution
    for d_idx = 1:size(naps, 1)

        for n_idx = 1:size(naps, 2)

            fig_name = ['naps_kde_PDF_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1}(1,:);
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate

            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            % Remove zeros in actual.pdf_y generated from inf/nan values
            [row,~] = find(actual.pdf_y > 0);
            for s_idx = 1:trials
                nmem_h = plot(naps_kde{d_idx,n_idx,s_idx,3}, naps_kde{d_idx,n_idx,s_idx,4},'-', 'DisplayName', '$\hat{f}_i^{NMEM}(x)$');
            end
            g = plot(actual.x(row,1), actual.pdf_y(row,1), '-k', 'DisplayName','$f(x)$');
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{f}(x)$','Interpreter','latex')
            if ismember(actual.dist_name, ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5"])
                xlim([0,1])
                ylim([0,10])
            else
                xlim([0,10])
                ylim([0,1])
            end
            legend([g],'Interpreter','latex')
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end

            fig_name = ['kde_PDF_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1}(1,:);
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate

            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            % Remove zeros in actual.pdf_y generated from inf/nan values
            [row,~] = find(actual.pdf_y > 0);
            for s_idx = 1:trials
                nmem_h = plot(kde{d_idx,n_idx,s_idx,3}, kde{d_idx,n_idx,s_idx,4},'-', 'DisplayName', '$\hat{f}_i^{NMEM}(x)$');
            end
            g = plot(actual.x(row,1), actual.pdf_y(row,1), '-k', 'DisplayName','$f(x)$');
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{f}(x)$','Interpreter','latex')
            if ismember(actual.dist_name, ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5"])
                xlim([0,1])
                ylim([0,10])
            else
                xlim([0,10])
                ylim([0,1])
            end
            legend([g],'Interpreter','latex')
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
            end

            fig_name = ['nmem_PDF_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1}(1,:);
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate

            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            % Remove zeros in actual.pdf_y generated from inf/nan values
            [row,~] = find(actual.pdf_y > 0);
            for s_idx = 1:trials
                nmem_h = plot(nmem{d_idx,n_idx,s_idx,3}, nmem{d_idx,n_idx,s_idx,4},'-', 'DisplayName', '$\hat{f}_i^{NMEM}(x)$');
            end
            g = plot(actual.x(row,1), actual.pdf_y(row,1), '-k', 'DisplayName','$f(x)$');
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{f}(x)$','Interpreter','latex')
            if ismember(actual.dist_name, ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5"])
                xlim([0,1])
                ylim([0,10])
            else
                xlim([0,10])
                ylim([0,1])
            end
            legend([g],'Interpreter','latex')
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end


            fig_name = ['naps_PDF_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1}(1,:);
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate

            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            % Remove zeros in actual.pdf_y generated from inf/nan values
            [row,~] = find(actual.pdf_y > 0);
            for s_idx = 1:trials
                naps_h = plot(naps{d_idx,n_idx,s_idx,3}, naps{d_idx,n_idx,s_idx,4},'-', 'DisplayName', '$\hat{f}_i^{NAP}(x)$');
            end
            g = plot(actual.x(row,1), actual.pdf_y(row,1), '-k', 'DisplayName','$f(x)$');
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{f}(x)$','Interpreter','latex')
            if ismember(actual.dist_name, ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5"])
                xlim([0,1])
                ylim([0,10])
            else
                xlim([0,10])
                ylim([0,1])
            end
            legend([g],'Interpreter','latex')
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
            end


            fig_name = ['combined_PDF_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1}(1,:);
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate

            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            % Remove zeros in actual.pdf_y generated from inf/nan values
            [row,~] = find(actual.pdf_y > 0);
            for s_idx = 1:trials
                nmem_h = plot(nmem{d_idx,n_idx,s_idx,3}, nmem{d_idx,n_idx,s_idx,4},'-b', 'DisplayName', '$\hat{f}_i^{NMEM}(x)$');
            end
            for s_idx = 1:trials
                naps_h = plot(naps{d_idx,n_idx,s_idx,3}, naps{d_idx,n_idx,s_idx,4},'-r', 'DisplayName', '$\hat{f}_i^{NAP}(x)$');
            end
            g = plot(actual.x(row,1), actual.pdf_y(row,1), '-k', 'DisplayName','$f(x)$');
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{f}(x)$','Interpreter','latex')
            if ismember(actual.dist_name, ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5"])
                xlim([0,1])
                ylim([0,10])
            else
                xlim([0,10])
                ylim([0,1])
            end
            legend([naps_h(1),nmem_h(1), g],'Interpreter','latex')
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end
        end
    end
end

%%% CDFs %%%
if plot_cdf
    % Plot data the order of indices corresponds to distribution, sample size,
    % trial. Plot all trials per sample per distrobution
    for d_idx = 1:size(naps, 1)

        for n_idx = 1:size(naps, 2)

            fig_name = ['CDF_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1};
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate
            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            % Remove zeros in actual.pdf_y generated from inf/nan values
            [row,~] = find(actual.pdf_y > 0);

            for s_idx = 1:trials
                nmem_h = plot(nmem{d_idx,n_idx,s_idx,3}, nmem{d_idx,n_idx,s_idx,5},'-b', 'DisplayName', '$\hat{F}_i^{NMEM}(x)$');
            end
            for s_idx = 1:trials
                naps_h = plot(naps{d_idx,n_idx,s_idx,3}, naps{d_idx,n_idx,s_idx,5},'-r', 'DisplayName', '$\hat{F}_i^{NAP}(x)$');
            end
            for s_idx = 1:trials
                naps_x = naps{d_idx,n_idx,s_idx,3};
                naps_pdf = naps{d_idx,n_idx,s_idx,4};
            end
            g = plot(actual.x(row,1), actual.cdf_y(row,1), '-k', 'DisplayName','$F(x)$');
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{F}(x)$','Interpreter','latex')
            if ismember(actual.dist_name, ["Beta-a0p5-b1p5","Beta-a2-b0p5","Beta-a0p5-b0p5"])
                xlim([0,1])
            else
                xlim([0,10])
            end
            ylim([0,1])
            legend([naps_h(1),nmem_h(1), g],'Interpreter','latex')
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end
        end
    end
end

%%% SQRs %%%

if plot_sqr

    sqr_trials = 20;
    % Plot data the order of indices corresponds to distribution, sample size,
    % trial. Plot all trials per sample per distrobution
    for d_idx = 1:size(naps, 1)

        for n_idx = 1:size(naps, 2)

            fig_name = ['nmem_SQR_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(sqr_trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            lemondrop();
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1};
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate
            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            for s_idx = 1:sqr_trials
                nmem_h = plot(nmem{d_idx,n_idx,s_idx,6}, nmem{d_idx,n_idx,s_idx,7},'-', 'DisplayName', '$\hat{SQR}_i^{NMEM}(x)$');
            end
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{SQR}(x)$','Interpreter','latex')
            xlabel('$u$','Interpreter','latex')
            ylabel('$SQR(u)$','Interpreter','latex')
            xlim([0,1])
            ylim([-3,3])
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end


            fig_name = ['naps_SQR_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(sqr_trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            lemondrop();
            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1};
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate
            actual.x = naps{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            for s_idx = 1:sqr_trials
                naps_h = plot(naps{d_idx,n_idx,s_idx,6}, naps{d_idx,n_idx,s_idx,7},'-', 'DisplayName', '$\hat{SQR}_i^{NAP}(x)$');
            end
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{SQR}(x)$','Interpreter','latex')
            xlabel('$u$','Interpreter','latex')
            ylabel('$SQR(u)$','Interpreter','latex')
            xlim([0,1])
            ylim([-3,3])
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end

            fig_name = ['kde_SQR_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(sqr_trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            lemondrop();
            % Set actual distribution type
            actual.dist_name = kde{d_idx,n_idx,1,1};
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate
            actual.x = kde{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);

            for s_idx = 1:sqr_trials
                naps_h = plot(kde{d_idx,n_idx,s_idx,6}, kde{d_idx,n_idx,s_idx,7},'-', 'DisplayName', '$\hat{SQR}_i^{NAP}(x)$');
            end
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{SQR}(x)$','Interpreter','latex')
            xlabel('$u$','Interpreter','latex')
            ylabel('$SQR(u)$','Interpreter','latex')
            xlim([0,1])
            ylim([-3,3])
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end

            fig_name = ['naps_kde_SQR_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(sqr_trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;
            lemondrop();
            % Set actual distribution type
            actual.dist_name = naps_kde{d_idx,n_idx,1,1};
            % Set distrobution over specific range i.e. the x values from the
            % first trial estimate
            actual.x = naps_kde{d_idx,n_idx,1,3}(:,1);
            % Generate distrobution attributes
            actual = dist_list(actual);
            % Remove zeros in actual.pdf_y generated from inf/nan values
            [row,~] = find(actual.pdf_y > 0);

            for s_idx = 1:sqr_trials
                naps_h = plot(naps_kde{d_idx,n_idx,s_idx,6}, naps_kde{d_idx,n_idx,s_idx,7},'-', 'DisplayName', '$\hat{SQR}_i^{NAP}(x)$');
            end
            bp = gca;
            xlabel('$x$','Interpreter','latex')
            ylabel('$\hat{SQR}(x)$','Interpreter','latex')
            xlabel('$u$','Interpreter','latex')
            ylabel('$SQR(u)$','Interpreter','latex')
            xlim([0,1])
            ylim([-3,3])
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end
        end
    end
end

%%% Heavy tails %%%

if plot_heavy
    % Plot data the order of indices corresponds to distribution, sample size,
    % trial. Plot all trials per sample per distrobution
    disp('Calculate Heavy Tails')
    for d_idx = 1:size(naps, 1)

        for n_idx = 1:size(naps, 2)

            fig_name = ['Heavy_Tails_d',convertStringsToChars(d_vec(d_idx)),num2str(d_idx),'_s_',num2str(n_vec(n_idx)),'_t_',num2str(trials)];
            figure('Name',fig_name, 'visible',fig_plot)
            hold on;

            % Set actual distribution type
            actual.dist_name = naps{d_idx,n_idx,1,1};

            for s_idx = 1:trials

                % track calculations
                disp(['dist: ',num2str(d_idx),'/',num2str(length(d_vec)),...
                    ' s: ',num2str(n_idx),'/',num2str(length(n_vec)),...
                    ' t: ',num2str(s_idx),'/',num2str(trials)])

                % Set distrobution over specific range i.e. the x values from the
                % first trial estimate
                xs = naps{d_idx,n_idx,1,3}(:,1);
                x_min = min(xs);
                x_max = max(xs);

                actual.x = linspace(x_min,x_max*2,1000);
                % Generate distrobution attributes
                actual = dist_list(actual);

                [naps_htx, naps_hty, naps_htx_act, naps_hty_act] = analysis.heavy(naps{d_idx,n_idx,s_idx,3}(:,j),...
                    naps{d_idx,n_idx,s_idx,5}(:,j), actual, 'log');

                naps_h = plot(real(naps_htx), real(naps_hty), '-r', 'DisplayName', '$\hat{F}_i^{NAPS}(x)$');

                [nmem_htx, nmem_hty, nmem_htx_act, nmem_hty_act] = analysis.heavy(nmem{d_idx,n_idx,s_idx,3}(:,j),...
                    nmem{d_idx,n_idx,s_idx,5}(:,j), actual, 'log');

                nmem_h = plot(real(nmem_htx), real(nmem_hty), '-b', 'DisplayName', '$\hat{F}_i^{NMEM}(x)$');

                % update x-limits for actual cdf
                if s_idx == 1
                    x_min_act = min(naps_htx_act);
                    x_max_act = max(naps_htx_act);
                else
                    if min(naps_htx_act) < x_min_act
                        x_min_act = min(naps_htx_act);
                    end
                    if max(naps_htx_act) > x_max_act
                        x_max_act = max(naps_htx_act);
                    end
                end
            end

            actual.x = linspace(x_min_act*2,x_max_act*2,1000);
            % generate distribution attributes
            actual = dist_list(actual);

            htx_act = log10(actual.x);
            hty_act = log10(1-actual.cdf_y);

            g = plot(naps_htx_act,naps_hty_act,'-k', 'DisplayName','$F(x)$');
            bp = gca;
            xlabel('log$_{10}(x)$','Interpreter','latex')
            ylabel('log$_{10}(1 - \hat{F}(x))$','Interpreter','latex')
            legend([nmem_h(1),naps_h(1), g],'Interpreter','latex', 'Location','southwest')
            if save_figs
                saveas(bp, fullfile(fig_dir, [fig_name, '.png']))
                saveas(bp, fullfile(fig_dir, [fig_name, '.fig']))
            end
        end
    end
end


