
function process_func_naps(distribution_vector,names,trials, min_pow,max_pow,cpu_n,read_path)

addpath("functions/")

% JOB ARRAY VARIABLES -----------------------------------------------------

% cast characters to strings or to ints for relevant variables
distribution_vector = string(distribution_vector)
names = string(names)
trials = str2num(trials)
min_pow = str2num(min_pow)
max_pow = str2num(max_pow)
cpu_n = str2num(cpu_n)
read_path = string(read_path)

%%%%%%%%%%%%%%%%% COMMENT OUT IF RUNNING LOCALLY %%%%%%%%%%%%%%%%%%%%%%%%%%

% % create parallel process on cluster
% 
% % create a local cluster object
% % pc = parcluster('local')
% 
% % explicitly set the JobStorageLocation to the temp directory that was
% % created in your sbatch script
% % pc.JobStorageLocation = strcat('/home/zmerino/.matlab/temp_cluster_jobs/', getenv('SLURM_JOB_ID'))
% 
% % start the matlabpool with maximum available workers
% % control how many workers by setting ntasks in your sbatch script
% poolobj = gcp('nocreate'); % get the pool object, and do it avoiding creating a new one.
% if isempty(poolobj) % check if there is not a pool.
%     % create parallel process with given parameters
% %     parpool(pc, str2num(getenv('SLURM_CPUS_ON_NODE')), 'IdleTimeout', Inf)
%     parpool('Processes', str2num(getenv('SLURM_CPUS_ON_NODE')), 'IdleTimeout', Inf)
% else
%     delete( gcp('nocreate')); % delete the current pool object.
% %     % create parallel process with given parameters
%     parpool(pc, str2num(getenv('SLURM_CPUS_ON_NODE')), 'IdleTimeout', Inf)
%     parpool('Processes', str2num(getenv('SLURM_CPUS_ON_NODE')), 'IdleTimeout', Inf)
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

calc_q = false;

%%%%%%%%%%%%%%%%%%%%%%%%%%% Import data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Trials per sample
trials = 100;
plt_trials = 10;
% number of quantile segments
qnum = 50;
% number of cpus used to compute raw data
% cpu_n = 20;

% Create actual distrobution onjects
actual = distributions;

% Path to directory
% dir_name = fullfile('data','pdf_estimates');
table_name = 'mse_kl_100.dat';
% table_name = sprintf('%s_mse_kl_cpu_%d_t_%d.dat', distribution_vector, cpu_n, trials)
% dir_name = fullfile(strcat('data_cpu_',num2str(cpu_n),'_wall_2'));
dir_name = fullfile(read_path);
write_dir = fullfile(dir_name,strcat('kl_mse_cpu_',num2str(cpu_n),'_t_',num2str(trials)));
status = mkdir(write_dir);

% Define the etimates to plot

% Sample range
n_vec = 2.^[min_pow:max_pow];
d_vec = distribution_vector;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% quantiles %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

actual = distributions;
actual.generate_data = false;

% Initialize empty structure to store data in
mse_dist_naps_per_q = zeros(qnum,plt_trials);
mpe_dist_naps_per_q = zeros(qnum,plt_trials);

global_table = table();

for i = 1:length(d_vec)


    % save distribution name to object
    dist_name = d_vec(i);

    for j = 1:length(n_vec)
%         parfor k = 1:trials
        for k = 1:trials

            actual = distributions;
            actual.generate_data = false;

            % save distribution name to object
            actual.dist_name = dist_name;

            % Make sure that distribution names are saved as character vectors

            % track calculations
            disp(['dist: ',num2str(i),'/',num2str(length(d_vec)),...
                ' s: ',num2str(j),'/',num2str(length(n_vec)),...
                ' t: ',num2str(k),'/',num2str(trials)])

            % Load data from estimate directory
            read_dir = fullfile(dir_name, strcat(d_vec(i),'_cpu_',num2str(cpu_n),'_t_',num2str(trials)));
            naps = load(fullfile(read_dir,['naps_',num2str(d_vec(i)),'_t_',num2str(trials),'_s_', num2str(n_vec(j)),'.mat']));

            % x
            naps_data = naps.naps_pdf_data;
            clear naps

            naps_x = naps_data{2,k}'; % make sure column vector

            % pdf
            naps_pdf = naps_data{3,k}'; % make sure column vector

            % cdf
            naps_cdf = analysis.trapz(naps_data{2,k}, naps_data{3,k});

            % calcualte mpe,mse,quantile ----------------------------------

            % NAP ---------------------------------------------------------
            xs = naps_x;
            fs = naps_pdf;
            Fs = naps_cdf;

            clear naps_data

            % get actual distribution
            actual.min_limit = min(xs);
            actual.max_limit = max(xs);
            actual.x = xs;
            actual = actual.dist_list();

            % MPE per quantile
            if calc_q
                [mpe_dist_naps_per_q(:,k), ~,~,~] = analysis.quant_metric(xs, fs, Fs, qnum, d_vec(i),'MPE');
            end
            % MPE per trial
            mpe_dist_naps(j, k) = analysis.mpe(n_vec(j), actual.pdf_y, fs);


            % MSE per quantile
            if calc_q
                [mse_dist_naps_per_q(:,k),~,~,~] = analysis.quant_metric(xs, fs, Fs, qnum, d_vec(i),'MSE');
            end
            % MSE per trial
            mse_dist_naps(j, k) = analysis.mse(actual.pdf_y, fs);


        end

        if calc_q
            filename = sprintf('mse_per_q_%s_n_%s_t_%s.mat',d_vec(i), num2str(n_vec(j)), num2str(trials));
            save(fullfile(write_dir,['naps_', filename]), 'mse_dist_naps_per_q','-v7.3')

            filename = sprintf('mpe_per_q_%s_n_%s_t_%s.mat',d_vec(i), num2str(n_vec(j)), num2str(trials));
            save(fullfile(write_dir,['naps_', filename]), 'mpe_dist_naps_per_q','-v7.3')
        end


    end

    % build table

    % MPE
    temp = utils.reshape_groups(n_vec',mpe_dist_naps);
    mpe_total = temp(:,2);

    % MSE
    temp = utils.reshape_groups(n_vec',mse_dist_naps);

    sample_power = temp(:,1);
    mse_total = temp(:,2);

    % Distributions
    distribution = repelem(d_vec(i), length(temp(:,2)))';
    name = repelem(names(i), length(temp(:,2)))';

    naps_label = repelem(["NAPS(NMEM)"], size(utils.reshape_groups(n_vec',mse_dist_naps), 1));

    estimator = naps_label';


    dist_table = table(distribution, name, estimator, sample_power, mse_total,mpe_total);
    dist_table = splitvars(dist_table);

    % append table per distribution to global table containing data
    % for all distributions
    global_table = [global_table; dist_table];
end


writetable(global_table,fullfile(write_dir,table_name),"WriteMode","append")
end